package com.xynet.pay.util;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;

/**
 * HMAC_SHA1 Sign生成器. 
 *
 * 需要apache.commons.codec包 
 *
 */
public class HMAC_SHA1Utils {

    private static final String HMAC_SHA1_ALGORITHM = "HmacSHA1";

    /**
     * 使用 HMAC-SHA1 签名方法对data进行签名 
     *
     * @param data
     *            被签名的字符串 
     * @param key
     *            密钥      
     * @return
    加密后的字符串
     */
    public static String genHMAC(String data, String key) {
        byte[] result = null;
        try {
            //根据给定的字节数组构造一个密钥,第二参数指定一个密钥算法的名称    
            SecretKeySpec signinKey = new SecretKeySpec(key.getBytes(), HMAC_SHA1_ALGORITHM);
            //生成一个指定 Mac 算法 的 Mac 对象    
            Mac mac = Mac.getInstance(HMAC_SHA1_ALGORITHM);
            //用给定密钥初始化 Mac 对象    
            mac.init(signinKey);
            //完成 Mac 操作     
            byte[] rawHmac = mac.doFinal(data.getBytes());
            result = Base64.encodeBase64(rawHmac);

        } catch (NoSuchAlgorithmException e) {
            System.err.println(e.getMessage());
        } catch (InvalidKeyException e) {
            System.err.println(e.getMessage());
        }
        if (null != result) {
            return new String(result);
        } else {
            return null;
        }
    }
    /**
     * 测试 
     * @param args
     */
    /*public static void main(String[] args) {
        String genHMAC = genHMAC("111", "2222");
        System.out.println(genHMAC.length()); //28
        System.out.println(genHMAC);  // O5fviq3DGCB5NrHcl/JP6+xxF6s=
    }*/

    public static void main(String[] args){//OmmcT/30A1TyGAxjNE/9bdM0OAk=
        String key = "YTVhNzUxYzAtOWU3My00NTM5LWFlYWQtNjFiMjhlOGEwZWI2MjAyMDA1MTgxMTEx";
        String secret = "NDdkYWE3ZTMtOTM3NC00YWQ1LWE1ZWMtNGRkOTg1MzVkMzk5MjAyMDA1MTgxMTEx";
        Map map = new HashMap();
        List<Map> invoices = new ArrayList<>();
        Date date = new Date();
        Map invoices_map = new HashMap();
        invoices_map.put("order_id","200519140100001433");
        invoices_map.put("invoice_date",DateUtil.dateToString3(date,"yyyyMMdd",0));
        invoices_map.put("invoice_time",DateUtil.dateToString3(date,"HHmmss",0));
        invoices_map.put("tax_type","3");
        invoices_map.put("tax_amount",0);
        invoices_map.put("sales_amount",0);
        invoices_map.put("tax_rate",0.05);
        invoices_map.put("free_tax_sales_amount",0);
        invoices_map.put("zero_tax_sales_amount",0);
        invoices_map.put("total_amount",1234);
        invoices_map.put("main_remark","测试");
        invoices_map.put("print_mark","Y");
        invoices_map.put("random_number","AAAA");

        Map buyer = new HashMap();
        buyer.put("identifier","00000000");
        buyer.put("name","兴元网络");
        buyer.put("email_address","282929986@qq.com");
        invoices_map.put("buyer",buyer);
        invoices.add(invoices_map);
        map.put("invoices",invoices);
        List<Map> details1 = new ArrayList<>();
        Map details = new HashMap();
        details.put("description","FOOD");
        details.put("quantity",1);
        details.put("sequence_number","1");
        details.put("unit_price",1234);
        details.put("amount",1234);
        details1.add(details);
        invoices_map.put("details",details1);

       String data = "{\n" +
                "\t\"invoices\": [{\n" +
                "\t\t\"order_id\": \"200519140100001434\",\n" +
                "\t\t\"buyer\": {\n" +
                "\t\t\t\"identifier\": \"53567686\",\n" +
                "\t\t\t\"name\": \"兴元网络\"\n" +
                //"\t\t\t\"email_address\": \"282929986@qq.com\"\n" +
                "\t\t},\n" +
                "\t\t\"invoice_date\": \"20200520\",\n" +
                "\t\t\"invoice_time\": \"142022\",\n" +
                "\t\t\"tax_type\": \"1\",\n" +
                "\t\t\"tax_rate\": 0.5,\n" +
                "\t\t\"tax_amount\": 24,\n" +
                "\t\t\"sales_amount\": 1234,\n" +
                "\t\t\"free_tax_sales_amount\": 12,\n" +
                "\t\t\"zero_tax_sales_amount\": 123,\n" +
                "\t\t\"total_amount\": 1234,\n" +
                "\t\t\"print_mark\": \"Y\",\n" +
                "\t\t\"random_number\": \"AAAA\",\n" +
                "\t\t\"details\": [{\n" +
                "\t\t\t\"description\": \"FOOD\",\n" +
                "\t\t\t\"quantity\": 1,\n" +
                "\t\t\t\"sequence_number\": \"1\",\n" +
                "\t\t\t\"unit_price\": 1000,\n" +
                "\t\t\t\"amount\": 245\n" +
                "\t\t}]\n" +
                "\t}]\n" +
                "}";

        long l = System.currentTimeMillis()/1000;
        Map<String, Object> params = new HashMap<>();
        params.put("api_key",key);
        params.put("timestamp",l+"");
        params.put("auto_assign_invoice_track","true");
        params.put("invoice",JsonUtils.objectToJson(map));

        TreeMap<String,Object> treeMap = new TreeMap();
        treeMap.put("api_key",key);
        treeMap.put("timestamp",l+"");
        treeMap.put("auto_assign_invoice_track","true");
        treeMap.put("invoice",JsonUtils.objectToJson(map));
        String data1 = "";

        for (Map.Entry<String, Object> entry : treeMap.entrySet()) {
            data1+=entry.getKey()+"="+entry.getValue()+"&";
        }
        String substring = data1.substring(0, data1.length() - 1);
        System.out.println(substring);
        String genHMAC = genHMAC(substring, secret);
        System.out.println(genHMAC.length()); //28
        System.out.println(genHMAC);  // O5fviq3DGCB5NrHcl/JP6+xxF6s=
        String url = "https://boxtest.ecloudlife.com/customer/api/C0401";
        System.out.println("参数:"+JsonUtils.objectToJson(params));
        String s = HttpServiceUtils.sendPost(url, params, genHMAC);
        System.out.println(s);
    }
} 